//functions to handles RENDERINFO, which is mostly a DIBSECTION
//we're almost object-oriented here
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "dibsect.h"

void RenderObjGetRect(RENDERINFO*pri,RECT*prc,BYTE*psw)
{WORD*pstart;
 int linew,linenum;

  linew=(prc->right - prc->left)<<1; //number of bytes
  pstart=pri->pSurface+ (prc->left) +
         (pri->BmpInfoHdr.biWidth)*(prc->top);
  linenum=prc->bottom - prc->top;
  while (linenum--) {
    CopyMemory(psw,pstart,linew);
    pstart+=pri->BmpInfoHdr.biWidth;
    psw+=linew;
  }
}
void RenderObjPutRect(RENDERINFO*pri,RECT*prc,BYTE*psw)
{WORD*pstart;
 int linew,linenum;

  linew=(prc->right - prc->left)<<1; //number of bytes
  pstart=pri->pSurface+ (prc->left) +
         (pri->BmpInfoHdr.biWidth)*(prc->top);
  linenum=prc->bottom - prc->top;
  while (linenum--) {
    CopyMemory(pstart,psw,linew);
    pstart+=pri->BmpInfoHdr.biWidth;
    psw+=linew;
  }
}

int RenderObjCreate(RENDERINFO*pri,long width,long hite)
{BITMAPINFOHEADER bi;
 WORD*psurface;
 HBITMAP dibsect;

  bi.biSize=sizeof(BITMAPINFOHEADER); 
  bi.biWidth=width;         bi.biHeight=-hite; //top-down format
  bi.biPlanes=1;            bi.biBitCount=16; 
  bi.biCompression=BI_RGB;  bi.biSizeImage=0x20000L; 
  bi.biXPelsPerMeter=0;     bi.biYPelsPerMeter=0; 
  bi.biClrUsed=0;           bi.biClrImportant=0; 
  dibsect=CreateDIBSection(NULL,(BITMAPINFO*)&bi,DIB_RGB_COLORS,
                           (VOID*)(&psurface),NULL,0);
  if (NULL==dibsect) return -1;

  pri->BmpInfoHdr=bi;
  pri->pSurface=psurface;
  pri->hDibSect=dibsect;
  return 0;
}

BOOL RenderObjDelete(RENDERINFO*pri)
{BOOL rb;

  rb=DeleteObject(pri->hDibSect);
  FillMemory(pri,sizeof(RENDERINFO),0);
  return rb;
}

int RenderObjCreateCopyScr(RENDERINFO*pri)
//create a render object with the same size as the screen
//and copy screen image into it
{long sx,sy;
 int reti;
 HDC scrdc,memdc;
 HWND scrwnd;
 HBITMAP oldhb;

  sx=GetSystemMetrics(SM_CXSCREEN);
  sy=GetSystemMetrics(SM_CYSCREEN);
  reti=RenderObjCreate(pri,sx,sy);
  if (reti) return reti;
  
  scrwnd=GetDesktopWindow();
  scrdc=GetWindowDC(scrwnd);
  memdc=CreateCompatibleDC(scrdc);
  if (memdc) {
    oldhb=SelectObject(memdc,pri->hDibSect);
    BitBlt(memdc, 0,0,sx,sy, scrdc, 0,0, SRCCOPY);
    SelectObject(memdc,oldhb);
    DeleteDC(memdc);
  } else {
    RenderObjDelete(pri);
    reti=-1;
  }
  ReleaseDC(scrwnd,scrdc);
  return reti;
}

int RenderObjToDc(RENDERINFO*pri,HDC hdc,RECT*prc,int left,int top)
{int width,hite;

  width=prc->right-prc->left; hite=prc->bottom-prc->top;
  StretchDIBits(hdc,
                prc->left,prc->top,width,hite,
                left,-pri->BmpInfoHdr.biHeight-top-hite,width,hite,
                pri->pSurface,(BITMAPINFO*)&pri->BmpInfoHdr,
                DIB_RGB_COLORS,SRCCOPY);
  return 0;
}

int RenderObjToDc2(RENDERINFO*pri,HDC hdc,RECT*prc,int left,int top)
{HDC memdc;
 HBITMAP oldhb;
  
  memdc=CreateCompatibleDC(hdc);
  if (NULL==memdc) return -1;
  
  oldhb=SelectObject(memdc,pri->hDibSect);
  BitBlt(hdc, prc->left,prc->top,
         prc->right - prc->left, prc->bottom - prc->top,
         memdc, left,top,SRCCOPY);
  SelectObject(memdc,oldhb);

  DeleteDC(memdc);
  return 0;
}

