#define WIN32_LEAN_AND_MEAN
//based on MS fractal screensaver
#include <windows.h>
#include <scrnsave.h>
#include <stdlib.h>
#include "resource.h"
#include "fractal.h"
#include "util0.h"

extern DWORD WINAPI RenderProc(SSINFO*);
SSINFO  gInfo;

void ShutdownRenderThread(SSINFO*pinfo)
{
  if (pinfo->hThread) {
    pinfo->dwCommand=1;
#ifdef _DEBUG    
    DefMsgBox("waiting for thread exit");
#endif    
    WaitForSingleObject(pinfo->hThread,INFINITE);
    CloseHandle(pinfo->hThread);
  }
  FillMemory(pinfo,sizeof(SSINFO),0);
}

HANDLE StartRenderThread(SSINFO*pi,HWND hWnd)
{DWORD dwThreadId;

  FillMemory(pi,sizeof(SSINFO),0);

  pi->DrawWnd=hWnd;
  pi->hThread=CreateThread(NULL, 0, RenderProc, pi,
                STANDARD_RIGHTS_REQUIRED, &dwThreadId );
  if (pi->hThread) {
    SetThreadPriority(pi->hThread, THREAD_PRIORITY_BELOW_NORMAL);
  }
  
  return pi->hThread;
}

LONG APIENTRY ScreenSaverProc(HWND hWnd,UINT msg,WPARAM wParam,LPARAM lParam)
//main screen saver message loop
{
  switch (msg) {
    case WM_CREATE: 
      FillMemory(&gInfo,sizeof(SSINFO),0);
      break;

    case WM_ERASEBKGND: {
      if (gInfo.hThread) return 1; //if thread already created
      StartRenderThread(&gInfo,hWnd);
      return 1;
    }
    case WM_DESTROY:
      ShutdownRenderThread(&gInfo);
      CheckMemory();
      break;
  }
  return (DefScreenSaverProc(hWnd,msg,wParam,lParam));
}

BOOL WINAPI RegisterDialogClasses(HANDLE hInst) 
{//register window classes of screen saver configuration dialog
  return TRUE;
}

char szHelpTxt[]={
"Shadows Screensaver 1.0 by Bo Yang\015\012\
web: http://www.geocities.com/SiliconValley/Way/9877\015\012\
email: byang@geocities.com\015\012\
Last modified: Nov 12, 1997\015\012\
\015\012\
This screensaver runs only in high-color or true-color \
video modes, it does NOT work properly in a 256-color mode.\015\012\
\015\012\
This screensaver uses a 256-color BMP file to lighten or darken \
your screen. For every pixel in the BMP file, if the average \
of its RGB values (as indicated by the palette) are lower than \
medium gray (128), then the screen pixel beneath it will be darkened; \
if it's greater than medium gray then the screen pixel beneath it will \
be lightened. At medium gray the screen pixel remains unchanged. \
The amount of lightening or darkening depends on the average of \
BMP pixel's RGB values, ranging from total blackness (0) to \
total whiteness (255).\015\012\
\015\012\
The name of the BMP file is SHDWSS.BMP, and it must be in the same \
directory as SHDWSS.SCR (either windows directory or windows \
system directory). You can edit this file to get different \
patterns, but just remember:\015\012\
\015\012\
1. The file name must be SHDWSS.BMP.\015\012\
2. It must be a 256-color BMP.\015\012\
3. Its width must be a multiple of 4.\015\012\
4. Its minimum size is 64x64.\
"
};

BOOL APIENTRY ScreenSaverConfigureDialog(
  HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
//screen saver configuration dialog
{
  switch(message) {
    case WM_INITDIALOG: 
      SetDlgItemText(hDlg,IDC_HLPINFO,szHelpTxt);
#ifdef _DEBUG
      FillMemory(&gInfo,sizeof(SSINFO),0);
      ShowWindow(GetDlgItem(hDlg,IDC_BTNTEST),SW_SHOW);
#endif      
      return TRUE;
    
    case WM_CLOSE:
#ifdef _DEBUG
      if (gInfo.hThread) {
        ShutdownRenderThread(&gInfo);
        DestroyWindow(gInfo.DrawWnd);
      }
      CheckMemory();
#endif      
      EndDialog(hDlg,0);
      return TRUE;

    case WM_COMMAND:
      switch(LOWORD(wParam)) {
        case IDOK: 
          PostMessage(hDlg,WM_CLOSE,0,0);
          return TRUE;
#ifdef _DEBUG
        case IDC_BTNTEST:
          if (NULL==gInfo.hThread) {
            HWND swnd; RECT rc;
            GetWindowRect(GetDesktopWindow(),&rc);
            swnd=CreateWindowEx(WS_EX_TRANSPARENT,
              "STATIC","Test Saver",SS_SIMPLE, 
              rc.left,rc.top,rc.right-rc.left,rc.bottom-rc.top,
              NULL,NULL,
              (HINSTANCE)GetWindowLong(hDlg,GWL_HINSTANCE),0);
            if (swnd) {
              SetWindowLong(swnd,GWL_STYLE,
                            (~(WS_BORDER|WS_CAPTION))&
                            GetWindowLong(swnd,GWL_STYLE));
              ShowWindow(swnd,SW_SHOWMAXIMIZED);
              StartRenderThread(&gInfo,swnd);
            } else {
              DefMsgBox("Cannot create test window");
            }
          }
          return TRUE;
#endif      
        default: return FALSE;
      }
  } 
  return FALSE;
}

